/**********************************************************************
  ga_replace.c
 **********************************************************************

  ga_replace - Genetic algorithm replacement operators.
  Copyright 2000, Stewart Adcock <stewart@bellatrix.pcl.ox.ac.uk>

  The latest version of this program should be available at:
  http://www.stewart-adcock.co.uk/

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.  Alternatively, if your project
  is incompatible with the GPL, I will probably agree to requests
  for permission to use the terms of any other license.

  This program is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY WHATSOEVER.

  A full copy of the GNU General Public License should be in the file
  "COPYING" provided with this distribution; if not, see:
  http://www.gnu.org/

 **********************************************************************

  Synopsis:     Routines for performing GA replacement operations.

		These functions should duplicate user data were
		appropriate.

  To do:	Replace parents if less fit.
		Replace most similar entities.
		Replace 'oldest' entities.

 **********************************************************************/

#include "ga_core.h"

/**********************************************************************
  ga_replace_by_fitness()
  synopsis:	Replace by fitness.  (i.e. New entity gets inserted
		into population, according to fitness, and least fit
		entity dies.)
  parameters:
  return:
  last updated: 31/05/01
 **********************************************************************/

void ga_replace_by_fitness(population *pop, entity *child)
  {
  int		i, j;		/* Loop over entities. */
  entity	*tmp;		/* For swapping. */
  boolean	finished=FALSE;	/* Shuffling finished. */

  /* Find child's current rank, which will be somewhere near the bottom. */
  i=pop->size;
  do
    {
    i--;
    } while (i>=pop->orig_size && !(child == pop->entity_iarray[i]));

  if (i<pop->orig_size) die("Dodgy replacement.");

  if (child->fitness >= pop->entity_iarray[pop->orig_size-1]->fitness)
    {
    tmp = pop->entity_iarray[pop->orig_size-1];
    pop->entity_iarray[pop->orig_size-1] = child;
    child = tmp;

    /* Shuffle entity to rightful location. */
    j = pop->orig_size-1;
    while (j>0 && finished==FALSE)
      {
      if (pop->entity_iarray[j]->fitness > pop->entity_iarray[j-1]->fitness)
        {
        tmp = pop->entity_iarray[j];
        pop->entity_iarray[j] = pop->entity_iarray[j-1];
        pop->entity_iarray[j-1] = tmp;
        }
      else
        {
        finished=TRUE;
        }
      }
    i = pop->orig_size-1;
    }

  /* Kill off an entity. */
  ga_entity_dereference_by_rank(pop, i);

  return;
  }


